{ open Parser }

rule token = parse
[' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "/*"     { comment lexbuf }           (* Comments *)
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| '['      { LBRACKET }
| ']'      { RBRACKET }
| ';'      { SEMI }
| ','      { COMMA }
| '!'      { NOT }
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '%'      { MOD }
| '='      { ASSIGN }
| "=="     { EQ }
| "+="     { PLUSEQ }
| "-="     { MINUSEQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "&&"     { AND }
| "||"     { OR }
| ":="     { DECL }
| ":"      { COLON }
| "::"     { CONS }
| "::="    { CONSEQ }
| "<<"     { SHL }
| ">>"     { SHR}
| "bool"   { BOOL }
| "down"   { DOWN }
| "else"   { ELSE }
| "false"  { BOOLLITERAL(false) }
| "for"    { FOR }
| "half"   { HALF }
| "if"     { IF }
| "in"     { IN }
| "int"    { INT }
| "list"   { LIST }
| "note"   { NOTE }
| "to"     { TO }
| "true"   { BOOLLITERAL(true) }
| "up"     { UP }
| "void"   { VOID }
| "while"  { WHILE }
| "whole"  { WHOLE }
| "return" { RETURN }
| ['0'-'9']+ as lxm { LITERAL(int_of_string lxm) }
| ['A'-'G']['b' '#']?['0'-'8']? as lxm { NOTELITERAL(lxm) }
| ['a'-'z' 'H'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| '\"'[^'"']*'\"' as lxm
            { STRINGLITERAL(String.sub lxm 1 ((String.length lxm) - 2)) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }
